<?php

declare(strict_types=1);

namespace Dedi\SyliusDpdPickUpPlugin\Locator;

use Dedi\SyliusDpdPickUpPlugin\Model\Location;
use Dedi\SyliusDpdPickUpPlugin\Model\Pudo;
use Psr\Log\LoggerInterface;
use SimpleXMLElement;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;
use Symfony\Contracts\HttpClient\HttpClientInterface;

class PickUpLocator
{
    public const PICK_UP_LOCATOR_ENDPOINT = 'GetPudoList';

    private LoggerInterface $logger;
    private HttpClientInterface $client;
    private string $mypudoUrl;
    private string $carrier;
    private string $key;

    public function __construct(
        LoggerInterface $logger,
        HttpClientInterface $client,
        string $mypudoUrl,
        string $carrier,
        string $key
    ) {
        $this->logger = $logger;
        $this->client = $client;
        $this->mypudoUrl = $mypudoUrl;
        $this->carrier = $carrier;
        $this->key = $key;
    }

    /**
     * @return Pudo[]
     */
    public function __invoke(
        int $requestId,
        \DateTimeInterface $dateFrom,
        Location $preferredLocation
    ): array {
        try {
            $response = $this->client->request(
                'GET',
                sprintf('%s/%s', $this->mypudoUrl, self::PICK_UP_LOCATOR_ENDPOINT),
                [
                    'query' => [
                        'carrier' => $this->carrier,
                        'key' => $this->key,
                        'address' => $preferredLocation->address,
                        'zipCode' => $preferredLocation->zipCode,
                        'city' => $preferredLocation->city,
                        'countryCode' => $preferredLocation->countryCode,
                        'requestID' => $requestId,
                        'date_from' => $dateFrom->format('d/m/Y'),
                        'max_pudo_number' => 10,
                        'max_distance_search' => '',
                        'weight' => '',
                        'category' => '',
                        'holiday_tolerant' => '',
                    ],
                ],
            );

            $data = new SimpleXMLElement($response->getContent());

            return $this->buildPudos($data);
        } catch (TransportExceptionInterface $e) {
            $this->logger->critical('Error contacting DPD PickUp services', [
                'response' => (string) $response->getContent(),
            ]);

            return [];
        } catch (\Exception $e) {
            $this->logger->critical('Exception with DPD PickUp Data', [
                'message' => $e->getMessage(),
            ]);

            return [];
        }
    }

    /**
     * @return Pudo[]
     */
    private function buildPudos(SimpleXMLElement $data): array
    {
        $pudos = [];
        foreach ($data->PUDO_ITEMS->PUDO_ITEM as $item) {
            $pudos[] = Pudo::fromDPDResponse((array) $item);
        }

        return $pudos;
    }
}
